/**
 * $Revision: 51044 $
 * $Date: 2007-10-01 17:35:13 -0700 (Mon, 01 Oct 2007) $
 *
 * Copyright (C) 1999-2006 Jive Software. All rights reserved.
 *
 * This software is the proprietary information of Jive Software.
 * Use is subject to license terms.
 */

package org.jivesoftware.plugins.instantFeeds;

import org.apache.commons.httpclient.params.HttpConnectionParams;
import org.apache.commons.httpclient.protocol.SecureProtocolSocketFactory;
import org.jivesoftware.util.Log;

import javax.net.SocketFactory;
import javax.net.ssl.SSLSocketFactory;
import java.io.IOException;
import java.net.InetAddress;
import java.net.Socket;
import java.security.KeyManagementException;
import java.security.NoSuchAlgorithmException;
import java.security.cert.CertificateException;
import java.security.cert.CertificateExpiredException;
import java.security.cert.CertificateNotYetValidException;
import java.security.cert.X509Certificate;

/**
 * SSLSocketFactory that accepts any certificate chain and also accepts expired certificates.
 * Implements ProtocolSocketFactory so it can be used with Commons HttpClient natively.
 */
public class InstantFeedsDummySSLSocketFactory extends SSLSocketFactory implements SecureProtocolSocketFactory {

    private SSLSocketFactory factory;

    public InstantFeedsDummySSLSocketFactory() {

        try {
            javax.net.ssl.SSLContext sslcontent = javax.net.ssl.SSLContext.getInstance("TLS");
            sslcontent.init(null, // KeyManager not required
                    new javax.net.ssl.TrustManager[]{new DummyTrustManager()},
                    new java.security.SecureRandom());
            factory = sslcontent.getSocketFactory();
        }
        catch (NoSuchAlgorithmException e) {
            Log.error(e);
        }
        catch (KeyManagementException e) {
            Log.error(e);
        }
    }

    public static SocketFactory getDefault() {
        return new InstantFeedsDummySSLSocketFactory();
    }

    public Socket createSocket(Socket socket, String s, int i, boolean flag) throws IOException {
        return factory.createSocket(socket, s, i, flag);
    }

    public Socket createSocket(InetAddress inaddr, int i, InetAddress inaddr2, int j)
            throws IOException
    {
        return factory.createSocket(inaddr, i, inaddr2, j);
    }

    public Socket createSocket(InetAddress inaddr, int i) throws IOException {
        return factory.createSocket(inaddr, i);
    }

    public Socket createSocket(String s, int i, InetAddress inaddr, int j) throws IOException {
        return factory.createSocket(s, i, inaddr, j);
    }

    public Socket createSocket(String s, int i, InetAddress inaddr, int j,
            HttpConnectionParams httpConnectionParams)
            throws IOException
    {
        //NOTE: we are ignoring any passed HttpConnectionParams
        return factory.createSocket(s, i, inaddr, j);
    }

    public Socket createSocket(String s, int i) throws IOException {
        return factory.createSocket(s, i);
    }

    public Socket createSocket() throws IOException {
        return factory.createSocket();
    }

    public String[] getDefaultCipherSuites() {
        return factory.getSupportedCipherSuites();
    }

    public String[] getSupportedCipherSuites() {
        return factory.getSupportedCipherSuites();
    }

    private static class DummyTrustManager implements javax.net.ssl.X509TrustManager {

        public boolean isClientTrusted(X509Certificate[] cert) {
            return true;
        }

        public boolean isServerTrusted(X509Certificate[] cert) {
            try {
                cert[0].checkValidity();
                return true;
            }
            catch (CertificateExpiredException e) {
                return false;
            }
            catch (CertificateNotYetValidException e) {
                return false;
            }
        }

        public void checkClientTrusted(java.security.cert.X509Certificate[] x509Certificates,
                String s) throws CertificateException
        {
        }

        public void checkServerTrusted(java.security.cert.X509Certificate[] x509Certificates,
                String s) throws CertificateException
        {
        }

        public X509Certificate[] getAcceptedIssuers() {
            return new X509Certificate[0];
        }
    }
}
